<#
               DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
    TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
    0. You just DO WHAT THE FUCK YOU WANT TO.
#>

<#
    Script to export registry keys associated with a solidworks installation
    YOU MUST RUN AS ADMIN. REGULAR SILENTLY FAILS

    If your current directry is within a DROPBOX or ONEDRIVE share, temporarily
    disable it. Such programs try to lock access to the temp files files during
    the concatenation loop, and I don't know how to code around that.

    Output is placed into the current directory. So create folder and call from
    there with `..\..\..\swx-registry-export.ps1` or just run in-place and
    copy the output where you want it.

    Example (default delay of 500 msec)
    > ./swx-registry-export 2020

    Example (longer delay if default gives errors)
    > ./swx-registry-export 2020 2000
#>

param (
    # Install year
    [Parameter(Mandatory=$true)]
    [ValidateRange(2015,2030)]
    [Int]
    $yr = 2020,

    # Configurable hack to give regedit time to release the file
    [Parameter(Mandatory=$false)]
    [ValidateRange(0,2000)]
    [Int]
    $delay_ms = 1000
)

# /A: Export part of the Registry in ANSI mode
# /E: Export part of the Registry
#
# The subkey SOFTWARE of HKCU is capitalized (most are not), but that matches HKLM\SOFTWARE. Under HKLM, all subkeys are in all-caps.

$machine = $Env:COMPUTERNAME.ToLower()

$swVerKey = "SOLIDWORKS " + "$yr"

$exportedLm = "$machine-hklm-$yr-toplvl"
$exportedLmVer = "$machine-hklm-$yr"
$exportedLmWow = "$machine-hklm-wow-$yr-toplvl"
$exportedLmWowVer = "$machine-hklm-wow-$yr"
$exportedCu = "$machine-hkcu-$yr-toplvl"
$exportedCuVer = "$machine-hkcu-$yr"

$output = @($exportedLm,$exportedLmVer,$exportedLmWow,$exportedLmWowVer,$exportedCu,$exportedCuVer)

foreach ($out in $output) {
    if (Test-Path "$out.reg") {
        $msg = "Already-Exists: " + "$out.reg"
        Write-Error $msg -ErrorAction Stop
    }
}

# My Cheetah 2020 install has all but "General" under HKEY_Local_Machine
# My Cheetah 2020 install has all but "Security" under HKEY_Current_User
$topLvlKeys = @("Addins","Applications","General","IM","Licenses","Security")

$files = @()
$i=-1
foreach($key in $topLvlKeys) {
    $i = $i+1
    $file = "$exportedLm$i.reg"
    $files += $file
    # /a is undocumented. It does the export in ANSI mode, as an alternative to /e
    # If the particular subkey doesn't exist, no file is generated. So
    # further down I'll have Remove-Item silently continue if a file isn't there.
    regedit /a "$file" "HKey_LOCAL_MACHINE\SOFTWARE\SolidWorks\$key"
}

# Hack to give regedit time to release the file
Start-Sleep -Milliseconds $delay_ms

$masterfile = "$exportedLm.reg"
foreach ($file in $files) {
    If (Test-Path -Path $file) {
        Add-Content -Value $(Get-Content $file) -Path $masterfile
        Remove-Item $file
    }
}

regedit /a "$exportedLmVer.reg" "HKey_LOCAL_MACHINE\SOFTWARE\SolidWorks\$swVerKey"

# =========

$topLvlKeys = @("Common Install","IM","Licenses","Security")

$files = @()
$i=-1
foreach($key in $topLvlKeys) {
    $i = $i+1
    $file = "$exportedLmWow$i.reg"
    $files += $file
    regedit /a "$file" "HKey_LOCAL_MACHINE\SOFTWARE\WOW6432Node\SolidWorks\$key"
}

# Hack to give regedit time to release the file
Start-Sleep -Milliseconds $delay_ms

$masterfile = "$exportedLmWow.reg"
$files | foreach { Add-Content -Value $(Get-Content $_) -Path $masterfile}
foreach ($file in $files) {
    Remove-Item $file
}

regedit /a "$exportedLmWowVer.reg" "HKey_LOCAL_MACHINE\SOFTWARE\WOW6432Node\SolidWorks\$swVerKey"

# =======

$topLvlKeys = @("AddinsStartup","Copy Settings Wizard","Diagnostics","General","IM","Licenses")

$files = @()
$i=-1
foreach($key in $topLvlKeys) {
    $i = $i+1
    $file = "$exportedCu$i.reg"
    $files += $file
    regedit /a "$file" "HKey_CURRENT_USER\SOFTWARE\SolidWorks\$key"
}

# Hack to give regedit time to release the file
Start-Sleep -Milliseconds $delay_ms

$masterfile = "$exportedCu.reg"
$files | foreach { Add-Content -Value $(Get-Content $_) -Path $masterfile}
foreach ($file in $files) {
    Remove-Item $file
}

regedit /a "$exportedCuVer.reg" "HKey_CURRENT_USER\SOFTWARE\SolidWorks\$swVerKey"
